package jp.asciimw.androidbook.chapter2.andlogo.canvas;

import java.util.ArrayList;
import java.util.List;

import jp.asciimw.androidbook.chapter2.andlogo.R;
import jp.asciimw.androidbook.chapter2.andlogo.core.LogoCommandReceiver;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.Toast;

public class LogoSurfaceView extends SurfaceView implements SurfaceHolder.Callback, Runnable, LogoCommandReceiver {
    class TurtleTrack {
        TurtleTrack(Path path, Paint paint) {
            this.path = path;
            this.paint = paint;
        }

        Path path;
        Paint paint;
        float x;
        float y;
    }

    private Drawable drawable;
    private Turtle turtle;
    private SurfaceHolder holder;
    private Rect viewRect;
    private Thread thread;

    List<TurtleTrack> lines = new ArrayList<TurtleTrack>();

    public LogoSurfaceView(Context context) {
        super(context);
        initialize(context);
    }

    public LogoSurfaceView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        initialize(context);
    }

    public LogoSurfaceView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initialize(context);
    }

    public void initTurtle() {
         turtle = new Turtle(drawable, viewRect);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#showTurtle()
     */
    public void showTurtle() {
        turtle.show();
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#hideTurtle()
     */
    public void hideTurtle() {
        turtle.hide();
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#setPenColor(int)
     */
    public void setPenColor(int penColor) {
        turtle.setPenColor(penColor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#clearScreen()
     */
    public void clearScreen() {
        initTurtle();
        synchronized (lines) {
            lines.clear();
        }
    }

    private TurtleTrack getTurtleTrack(float x2, float y2) {
        PointF p = turtle.getCurrentPoint();
        Path path = new Path();
        path.moveTo(p.x, p.y);
        path.lineTo(p.x + x2, p.y + y2);
        return new TurtleTrack(path, turtle.getCurrentPaint());
    }

    void addLine(float x, float y) {
        synchronized (lines) {
            lines.add(getTurtleTrack(x, y));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#forward(float)
     */
    public void forward(float distance) {
        float pixelDistance = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, distance, getResources().getDisplayMetrics());
        float angle = (float) (turtle.getDirection() * Math.PI / 180.0f);
        float newX = (float) (pixelDistance * Math.cos(angle));
        float newY = (float) -(pixelDistance * Math.sin(angle));
        if (!turtle.isPenUp()) {
            addLine(newX, newY);
        }
        turtle.move(newX, newY);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#back(float)
     */
    public void back(float distance) {
        forward(-distance);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#right(float)
     */
    public void right(float turnAngle) {
        turtle.right(turnAngle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#left(float)
     */
    public void left(float turnAngle) {
        turtle.left(turnAngle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#penUp()
     */
    public void penUp() {
        turtle.penUp();
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#penDown()
     */
    public void penDown() {
        turtle.penDown();
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.asciimw.androidbook.chapter2.andlogo.LogoCommandReceiver#setPenSize(int)
     */
    public void setPenSize(int penSize) {
        turtle.setPenStrokeWidth(penSize);
    }

    private void initialize(Context context) {
        drawable = context.getResources().getDrawable(R.drawable.turtle);
        holder = getHolder();
        holder.addCallback(this);
    }
    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        viewRect = new Rect(0, 0, width, height);
        holder.setFixedSize(width, height);
        initTurtle();
    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        thread = new Thread(this);
        thread.start();
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread = null;
    }

    @Override
    public void run() {
        Canvas canvas = null;
        Paint p = new Paint();
        p.setColor(Color.WHITE);
        while (thread != null) {
            try {
                canvas = holder.lockCanvas();
                if (canvas != null) {
                    canvas.drawRect(0, 0, getWidth(), getHeight(), p);
                    synchronized (lines) {
                        for (TurtleTrack track : lines) {
                            canvas.drawPath(track.path, track.paint);
                        }
                        if (turtle != null) {
                            turtle.draw(canvas);
                        }
                    }
                }
            } finally {
                if (canvas != null)
                    holder.unlockCanvasAndPost(canvas);
            }
        }
    }

}
